@@ -11,6 +11,7 @@ import pefile
 import glob
 import traceback
 import requirements
+import requests
 import re
 import zipfile
 from xml.dom import minidom
@@ -18,6 +19,8 @@ import toml
 import re
 import ast
 import textwrap
+import distutils.core
+from urllib.parse import urlparse, urlunparse
 
 from . import utils as lib_utils
 from . import code_secrets as lib_code_secrets
@@ -637,47 +640,15 @@ def discover_python(args, localpath):
         except:
             logging.error("Unable to parse python dependencies")
             continue
-    """
+
     files = lib_utils.find_files(localpath, 'setup.py')
     for file_path in files:
-        #Parse setup.py and return args and keywords args to its setup
-        #function call
-
-        mock_setup = textwrap.dedent('''\
-        def setup(*args, **kwargs):
-            __setup_calls__.append((args, kwargs))
-        ''')
-        parsed_mock_setup = ast.parse(mock_setup, filename=file_path)
-        with open(file_path, 'rt') as setup_file:
-            parsed = ast.parse(setup_file.read())
-            for index, node in enumerate(parsed.body[:]):
-                if (
-                    not isinstance(node, ast.Expr) or
-                    not isinstance(node.value, ast.Call)
-                    #not isinstance(node.value, ast.Call) or
-                    #node.value.func.id != 'setup'
-                ):
-                    continue
-                parsed.body[index:index] = parsed_mock_setup.body
-                break
-
-        fixed = ast.fix_missing_locations(parsed)
-        codeobj = compile(fixed, file_path, 'exec')
-        local_vars = {}
-        global_vars = {'__setup_calls__': []}
-        cwd = os.getcwd()
-        os.chdir(os.path.dirname(file_path))
         try:
-            exec(codeobj, global_vars, local_vars)
-        except Exception:
-            # move on
-            logging.warning("Unable to parse python dependencies %s", file_path)
-            os.chdir(cwd)
+            setup = distutils.core.run_setup(file_path, stop_after="config")
+        except:
             continue
-        os.chdir(cwd)
-        reqs = global_vars['__setup_calls__'][0][1]['install_requires']
-        for r in reqs:
-            prod = r
+        req = setup.install_requires 
+        for prod in req:
             prod = prod.replace("==", " ")
             prod = prod.replace(">=", " ")
             prod = prod.replace("<=", " ")
@@ -687,7 +658,6 @@ def discover_python(args, localpath):
             prod = prod + " source:" + file_path
             if prod not in plist:
                 plist.append(prod)
-    """
 
     files = lib_utils.find_files(localpath, 'pyproject.toml')
     for file_path in files:
@@ -708,6 +678,32 @@ def discover_python(args, localpath):
                 prod = d + " " + ver + " source:" + file_path
                 if prod not in plist:
                     plist.append(prod)
+        if 'tool' in tdict and 'poetry' in tdict['tool'] and 'dev-dependencies' in tdict['tool']['poetry']:
+            for d in tdict['tool']['poetry']['dev-dependencies']:
+                ver = tdict['tool']['poetry']['dev-dependencies'][d]
+                if isinstance(ver, dict):
+                    ver = ver['version']
+                ver = cleanse_semver_version(ver)
+                prod = d + " " + ver + " source:" + file_path
+                if prod not in plist:
+                    plist.append(prod)
+
+        if 'build-system' in tdict:
+            for b in tdict['build-system']:
+                if b == 'requires':
+                    for prod in tdict['build-system']['requires']:
+                        prod = prod.replace("==", " ")
+                        prod = prod.replace(">=", " ")
+                        prod = prod.replace("<=", " ")
+                        prod = prod.replace("~=", " ")
+                        prod = prod.replace(">", " ")
+                        prod = prod.replace("<", " ")
+                        prod = prod.replace(",", "")
+                        prod = prod + " source:" + file_path
+                        print("Builddep: "+prod)
+                        if prod not in plist:
+                            plist.append(prod)
+
         if 'project' in tdict and 'dependencies' in tdict['project']: 
             for d in tdict['project']['dependencies']:
                 prod = d 
@@ -720,6 +716,7 @@ def discover_python(args, localpath):
                 prod = prod + " source:" + file_path
                 if prod not in plist:
                     plist.append(prod)
+
     return plist, None
 
 def LOWORD(dword):
@@ -869,7 +866,25 @@ def get_asset_id(args):
     asset_id = asset_id.replace(':','-')
     return asset_id
 
-def discover_inventory(args, localpath):
+def prepare_permalink_base(args, repo_chksum):
+    repo = args.repo
+    purl = urlparse(repo)
+    hn = purl.netloc.split('@')[-1]
+    repo = urlunparse((
+        purl.scheme,
+        hn,
+        purl.path,
+        purl.params,
+        purl.query,
+        purl.fragment
+    ))
+    if args.mode == "bitbucket":
+        permalink_base = repo.replace('.git','') + '/src/' + repo_chksum + '/'
+    else:
+        permalink_base = repo.replace('.git','') + '/blob/' + repo_chksum + '/'
+    return permalink_base
+
+def discover_inventory(args, localpath, base_path, repo_chksum=None):
     asset_name = None
     if args.assetname == None or args.assetname.strip() == "":
         asset_name = get_last_component(args.repo)
@@ -913,8 +928,37 @@ def discover_inventory(args, localpath):
     asset_data['tags'] = asset_tags
     if len(tech2prod_dict) > 0:
         asset_data['compliance_metadata'] = {"source_metadata": {"technology_products":tech2prod_dict, "shallow_technology_products":shallow_tech2prod_dict}}
-   
     lib_utils.update_tool_run_record()
+
+    if args.secrets_scan:
+        logging.info("Discovering secrets/sensitive information. This may take some time.")
+        secret_records = lib_code_secrets.scan_for_secrets(args, localpath, base_path)
+        asset_data['secrets'] = secret_records
+        if len(secret_records) > 0:
+            asset_tags.append('Secret')
+    lib_utils.update_tool_run_record()
+
+    code_issues = []
+    if args.sast:
+        logging.info("Performing static analysis. This may take some time.")
+        sast_records = sast.run_sast(args, localpath, base_path)
+        code_issues.extend(sast_records)
+        if len(sast_records) > 0:
+            asset_tags.append('SAST')
+    lib_utils.update_tool_run_record()
+
+    if args.iac_checks:
+        logging.info("Identifying infrastructure as code (IaC) issues. This may take some time.")
+        iac_records = iac.run_iac_checks(args, localpath, base_path)
+        code_issues.extend(iac_records)
+        if len(iac_records) > 0:
+            asset_tags.append('IaC')
+    lib_utils.update_tool_run_record()
+
+    if len(code_issues) > 0:
+        asset_data['sast'] = code_issues
+    if repo_chksum is not None:
+        asset_tags.append('PERMALINK_BASE:'+prepare_permalink_base(args, repo_chksum))
     return [ asset_data ]
 
 # Note this error routine assumes that the file was read-only and hence could not be deleted
@@ -925,6 +969,7 @@ def on_rm_error( func, path, exc_info):
 # note this only handles actual repos and not orgs (i.e. all repos in an org)
 def get_inventory_helper(args):
     path = None
+    repo_chksum = None
     if args.repo.startswith('http'):
         if os.path.isfile(GIT_PATH) == False:
             logging.error("git executable does not exist at [%s]", GIT_PATH)
@@ -932,7 +977,6 @@ def get_inventory_helper(args):
         dev_null_device = open(os.devnull, "w")
         path = tempfile.mkdtemp()
         base_path = path
-        new_repo = None
         logging.info("Cloning repo locally...")
         out = ""
         try:
@@ -949,6 +993,17 @@ def get_inventory_helper(args):
             logging.error('Error cloning repo locally')
             shutil.rmtree(path, onerror = on_rm_error)
             return None
+        try:
+            cmdarr = [GIT_PATH, '-C', path+'/.', 'rev-parse', 'HEAD']
+            #out = subprocess.check_output(cmdarr, stderr=dev_null_device)
+            out = subprocess.check_output(cmdarr)
+            repo_chksum = out.decode('utf-8').strip()
+            logging.debug("Repo checksum is [%s]", repo_chksum)
+        except Exception as e:
+            logging.error(out)
+            logging.error(str(e))
+            logging.error(traceback.format_exc())
+            logging.error('Error getting repo checksum')
     elif os.path.isdir(args.repo):
         path = args.repo
         base_path = os.path.abspath(path)
@@ -958,72 +1013,77 @@ def get_inventory_helper(args):
         logging.error('Not a valid repo')
         return None
 
-    assets = discover_inventory(args, path)
-    if args.secrets_scan:
-        logging.info("Discovering secrets/sensitive information. This may take some time.")
-        secret_records = lib_code_secrets.scan_for_secrets(args, path, base_path)
-        assets[0]['secrets'] = secret_records
-    lib_utils.update_tool_run_record()
+    assets = discover_inventory(args, path, base_path, repo_chksum)
 
-    code_issues = []
-    if args.sast:
-        logging.info("Performing static analysis. This may take some time.")
-        sast_records = sast.run_sast(args, path, base_path)
-        code_issues.extend(sast_records)
-    lib_utils.update_tool_run_record()
-
-    if args.iac_checks:
-        logging.info("Identifying infrastructure as code (IaC) issues. This may take some time.")
-        iac_records = iac.run_iac_checks(args, path, base_path)
-        code_issues.extend(iac_records)
-    lib_utils.update_tool_run_record()
-
-    if len(code_issues) > 0:
-        assets[0]['sast'] = code_issues
-
-    if args.repo.startswith('http'):
-        shutil.rmtree(path, onerror = on_rm_error)
+    # repo directory will be cleaned up in push_asset_to_TW(...)
+    for asset in assets:
+        asset['tw_base_path'] = base_path
+        if args.repo.startswith('http'):
+            asset['tw_delete_base_path'] = True
 
     if len(assets[0]['products']) == 0 and (assets[0].get('secrets') is None or len(assets[0]['secrets']) == 0) and (assets[0].get('sast') is None or len(assets[0]['sast']) == 0):
         logging.warning("Nothing to report")
+        if args.repo.startswith('http'):
+            # if there is no asset reported, then cleanup repo directory here itself (if cloned)
+            shutil.rmtree(base_path, onerror = on_rm_error)
+            # remove the tw_delete_base_path indicator since repo folder has been deleted
+            delete_base_path = assets[0].pop('tw_delete_base_path', False)
         if args.create_empty_asset is None or not args.create_empty_asset:
             return [] # if there are no products nor secrets then no assets to report
     return assets
 
 def get_inventory(args):
-    if args.gh_user:
-        owner = args.gh_user
-        dev_null_device = open(os.devnull, "w")
-        # check if 'gh' command is available
-        try:
-            cmdarr = ['which', 'gh']
-            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
-        except subprocess.CalledProcessError as e:
-            logging.error("[gh] command not found")
-            return None
-        # check if user is logged in 'gh'
-        try:
-            cmdarr = ['gh', 'auth', 'status']
-            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
-        except subprocess.CalledProcessError as e:
-            logging.error("Please login using [gh auth login] command to list repositories")
-            logging.debug("[gh auth status] command returned exit code [%s]", e.returncode)
-            return None
+    if hasattr(args, 'gh_access_token'):
         # list repo for org/user
-        try:
-            cmdarr = ['gh', 'repo', 'list', owner, '--json', 'name,url', '--limit', '65535']
-            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
-        except subprocess.CalledProcessError as e:
-            logging.error("Unable to list repos for [%s]", owner)
-            logging.debug("[gh repo list %s --json name,url] command returned %s", owner, e.output)
-            logging.debug("[gh repo list %s --json name,url] command returned exit code [%s]", owner, e.returncode)
-            return None
-        repos = json.loads(out.strip())
+        repo_api_url = "%s/orgs/%s/repos" % (args.gh_api_url, args.gh_identity)
+        headers = { "Accept": "application/vnd.github+json", "Authorization" : "Bearer "+args.gh_access_token}
+        response = requests.get(repo_api_url, headers=headers, verify=False)
+        if response is None or response.status_code != 200:
+            logging.warning("Could not list repositories for Github Enterprise org [%s]", args.gh_identity)
+            repo_api_url = "%s/users/%s/repos" % (args.gh_api_url, args.gh_identity)
+            response = requests.get(repo_api_url, headers=headers, verify=False)
+            if response is None or response.status_code != 200:
+                logging.error("Could not list repositories for Github Enterprise user [%s]", args.gh_identity)
+                logging.debug(response.text)
+                return None
+        repos = json.loads(response.content.strip())
         assets = []
         for repo in repos:
-            logging.info("Discovering repo [%s] as an asset", repo['name'])
-            args.repo = repo['url']
-            logging.info("Repo URL: %s", args.repo)
+            logging.info("Discovering repo [%s] as an asset", repo['html_url'])
+            args.repo = repo['html_url'].replace("//","//"+args.gh_access_token+"@")
+            logging.debug("Repo URL: %s", args.repo)
+            temp_assets = get_inventory_helper(args)
+            assets.extend(temp_assets)
+        return assets
+    elif hasattr(args, 'gl_access_token'):
+        repo_api_url = "https://%s/api/v4/projects?membership=1" % (args.gl_host)
+        headers = { "PRIVATE-TOKEN" : args.gl_access_token}
+        response = requests.get(repo_api_url, headers=headers, verify=False)
+        if response is None or response.status_code != 200:
+            logging.error("Could not list repositories for GitLab Enterprise using the given access token")
+            logging.debug(response.text)
+            return None
+        repos = json.loads(response.content.strip())
+        assets = []
+        for repo in repos:
+            logging.info("Discovering repo [%s] as an asset", repo['http_url_to_repo'])
+            args.repo = repo['http_url_to_repo'].replace("//","//gitlab-ci-token:"+args.gl_access_token+"@")
+            logging.debug("Repo URL: %s", args.repo)
+            temp_assets = get_inventory_helper(args)
+            assets.extend(temp_assets)
+        return assets
+    elif hasattr(args, 'bb_app_password'):
+        response = requests.get(args.bb_repo_url, auth=(args.bb_user, args.bb_app_password), verify=False)
+        if response is None or response.status_code != 200:
+            logging.error("Could not list repositories for Bitbucket using the given credentials")
+            logging.debug(response.text)
+            return None
+        repos = json.loads(response.content.strip())
+        assets = []
+        for repo in repos['values']:
+            logging.info("Discovering repo [%s] as an asset", repo['links']['html']['href'])
+            args.repo = repo['links']['html']['href'].replace("//","//"+args.bb_user+":"+args.bb_app_password+"@")
+            logging.debug("Repo URL: %s", args.repo)
             temp_assets = get_inventory_helper(args)
             assets.extend(temp_assets)
         return assets
