@@ -107,23 +107,38 @@ def run_script_on_host(args, host, script_path, script_args):
             exit_code = excpt.returncode
     return pkgout, exit_code
 
-def scp_get_file(ssh_client, from_path, to_path):
-    try:
-        scp_client = SCPClient(ssh_client.get_transport())
-        scp_client.get(from_path, to_path)
-        return True
-    except:
-        logging.info("Unknown error copying file [%s] from remote host", from_path)
-        return False
+def scp_put_file(host, from_path, to_path):
+    _scp_file_helper(host, from_path, to_path, True)
 
-def scp_put_file(ssh_client, from_path, to_path):
+def scp_get_file(host, from_path, to_path):
+    _scp_file_helper(host, from_path, to_path, False)
+
+def _scp_file_helper(host, from_path, to_path, put_flag=True):
+    assetid = host['assetid'] if host.get('assetid') is not None else host['hostname']
+    ret = False
     try:
+        ssh_client = get_ssh_client(host)
         scp_client = SCPClient(ssh_client.get_transport())
-        scp_client.put(from_path, recursive=True, remote_path=to_path)
-        return True
+        if put_flag == True: # put file to remote machine
+            scp_client.put(from_path, recursive=True, remote_path=to_path)
+        else:
+            scp_client.get(from_path, to_path)
+        scp_client.close()
+        ssh_client.close()
+        ret = True
+    except paramiko.ssh_exception.AuthenticationException as e:
+        logging.info("Authentication failed for asset [%s], host [%s]", assetid, host['hostname'])
+        logging.info("Exception: %s", e)
+    except paramiko.ssh_exception.SSHException as e:
+        logging.info("SSHException while connecting to asset [%s], host [%s]", assetid, host['hostname'])
+        logging.info("Exception: %s", e)
+    except socket.error as e:
+        logging.info("Socket error while connection to asset [%s], host [%s]", assetid, host['hostname'])
+        logging.info("Exception: %s", e)
     except:
-        logging.info("Unknown error copying file [%s] to remote host", from_path)
-        return False
+        logging.info("Unknown error running remote discovery for asset [%s], host [%s]: [%s]", assetid, host['hostname'], sys.exc_info()[0])
+    finally:
+        return ret
 
 # NOTE script_path is path of script file on remote host
 def run_remote_ssh_script(args, host, script_path, script_args):
@@ -217,7 +232,7 @@ def get_asset_type(os):
     elif "debian" in os:
         return "Debian"
     elif "amazon linux" in os:
-        return "Amazon Linux"
+        return "Amazon Linux AMI"
     elif "oracle linux" in os:
         return "Oracle Linux"
     elif "freebsd" in os:
@@ -234,6 +249,8 @@ def get_asset_type(os):
         return "Alpine Linux"
     elif "container-optimized os" in os:
         return "Google Container-Optimized OS"
+    elif "rocky linux" in os:
+        return "Rocky Linux"
     else:
         logging.error("Not a supported OS type [%s]" % os)
         return None
@@ -377,6 +394,26 @@ def requests_post(url, json):
                 continue
         return resp
 
+def requests_post_files(url, files):
+    rc = 0
+    st = 1
+    while True:
+        try:
+            resp =  requests.post(url, files=files, verify=get_requests_verify())
+            resp_status_code = resp.status_code
+        except requests.exceptions.RequestException as e:
+            logging.warn("Retry count [%s] got exception: [%s]", rc, str(e))
+            if rc >= 10:
+                logging.warn("Max retries exceeded....giving up...")
+                return None
+            else:
+                logging.warn("Sleeping for [%s] seconds...", st)
+                time.sleep(st)
+                rc = rc + 1
+                st = st * 2
+                continue
+        return resp
+
 def requests_put(url, json):
     rc = 0
     st = 1
@@ -452,8 +489,10 @@ def update_tool_run_record(status='RUNNING', observations=None):
     auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
     if status != 'RUNNING':
         logging.shutdown()
+        lfn_suffix = args.run_id if args.run_id is not None else args.mode
+        logfilename = "twigs_%s.log" % lfn_suffix
         observations = { } if observations is None else observations
-        observations['log_snippet'] = tail('twigs.log', args.encoding, 100)
+        observations['log_snippet'] = tail(logfilename, args.encoding, 100)
     put_payload = { "status": status, "observations": observations }
     response = requests_put(url + auth_data, put_payload)
     return response
@@ -491,3 +530,8 @@ def tail(fn, encoding, lines=20):
         all_read_text = ''.join(reversed(blocks))
         return '\n'.join(all_read_text.splitlines()[-total_lines_wanted:])
 
+def get_latest_version():
+    response = requests.get('https://pypi.org/pypi/twigs/json')
+    latest_version = response.json()['info']['version']
+    return latest_version
+
